@@ -158,15 +158,10 @@ static void
 panel_applet_frame_dbus_sync_menu_state (PanelAppletFrame *frame,
 					 gboolean          movable,
 					 gboolean          removable,
-					 gboolean          lockable,
-					 gboolean          locked,
 					 gboolean          locked_down)
 {
 	PanelAppletFrameDBus *dbus_frame = PANEL_APPLET_FRAME_DBUS (frame);
 
-	panel_applet_container_child_set (dbus_frame->priv->container,
-					  "locked", g_variant_new_boolean (lockable && locked),
-					  NULL, NULL, NULL);
 	panel_applet_container_child_set (dbus_frame->priv->container,
 					  "locked-down", g_variant_new_boolean (locked_down),
 					  NULL, NULL, NULL);
@@ -184,6 +179,18 @@ panel_applet_frame_dbus_popup_menu (PanelAppletFrame *frame,
 						 NULL, NULL, NULL);
 }
 
+static void
+panel_applet_frame_dbus_popup_edit_menu (PanelAppletFrame *frame,
+					 guint             button,
+					 guint32           timestamp)
+{
+	PanelAppletFrameDBus *dbus_frame = PANEL_APPLET_FRAME_DBUS (frame);
+
+	panel_applet_container_child_popup_edit_menu (dbus_frame->priv->container,
+						      button, timestamp,
+						      NULL, NULL, NULL);
+}
+
 static void
 change_orientation_cb (PanelAppletContainer *container,
 		       GAsyncResult         *res,
@@ -315,14 +322,6 @@ panel_applet_frame_dbus_applet_move (PanelAppletContainer *container,
 	_panel_applet_frame_applet_move (frame);
 }
 
-static void
-panel_applet_frame_dbus_applet_lock (PanelAppletContainer *container,
-				     gboolean              locked,
-				     PanelAppletFrame     *frame)
-{
-	_panel_applet_frame_applet_lock (frame, locked);
-}
-
 static void
 panel_applet_frame_dbus_finalize (GObject *object)
 {
@@ -364,9 +363,6 @@ panel_applet_frame_dbus_init (PanelAppletFrameDBus *frame)
 	g_signal_connect (container, "applet-move",
 			  G_CALLBACK (panel_applet_frame_dbus_applet_move),
 			  frame);
-	g_signal_connect (container, "applet-lock",
-			  G_CALLBACK (panel_applet_frame_dbus_applet_lock),
-			  frame);
 }
 
 static void
@@ -380,6 +376,7 @@ panel_applet_frame_dbus_class_init (PanelAppletFrameDBusClass *class)
 	frame_class->init_properties = panel_applet_frame_dbus_init_properties;
 	frame_class->sync_menu_state = panel_applet_frame_dbus_sync_menu_state;
 	frame_class->popup_menu = panel_applet_frame_dbus_popup_menu;
+	frame_class->popup_edit_menu = panel_applet_frame_dbus_popup_edit_menu;
 	frame_class->change_orientation = panel_applet_frame_dbus_change_orientation;
 	frame_class->change_size = panel_applet_frame_dbus_change_size;
 	frame_class->change_background = panel_applet_frame_dbus_change_background;
@@ -411,6 +408,8 @@ panel_applet_frame_dbus_load (const gchar                 *iid,
 	PanelAppletFrameDBus *dbus_frame;
 	PanelAppletFrame     *frame;
 	GVariantBuilder       builder;
+	GdkScreen            *screen;
+	gchar                *settings_path;
 	gchar                *conf_path;
 	gchar                *background;
 	guint                 orient;
@@ -425,13 +424,18 @@ panel_applet_frame_dbus_load (const gchar                 *iid,
 	frame = PANEL_APPLET_FRAME (dbus_frame);
 	_panel_applet_frame_set_iid (frame, iid);
 
+	screen = panel_applet_frame_activating_get_screen (frame_act);
 	orient = get_panel_applet_orient (panel_applet_frame_activating_get_orientation (frame_act));
+	settings_path = panel_applet_frame_activating_get_settings_path (frame_act);
 	conf_path = panel_applet_frame_activating_get_conf_path (frame_act);
 	/* we can't really get a background string at this point since we don't
 	 * know the position of the applet */
 	background = NULL;
 
 	g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
+	g_variant_builder_add (&builder, "{sv}",
+			       "settings-path",
+			       g_variant_new_string (settings_path));
 	g_variant_builder_add (&builder, "{sv}",
 			       "prefs-key",
 			       g_variant_new_string (conf_path));
@@ -441,9 +445,6 @@ panel_applet_frame_dbus_load (const gchar                 *iid,
 	g_variant_builder_add (&builder, "{sv}",
 			       "size",
 			       g_variant_new_uint32 (panel_applet_frame_activating_get_size (frame_act)));
-	g_variant_builder_add (&builder, "{sv}",
-			       "locked",
-			       g_variant_new_boolean (panel_applet_frame_activating_get_locked (frame_act)));
 	g_variant_builder_add (&builder, "{sv}",
 			       "locked-down",
 			       g_variant_new_boolean (panel_applet_frame_activating_get_locked_down (frame_act)));
@@ -456,12 +457,13 @@ panel_applet_frame_dbus_load (const gchar                 *iid,
 	g_object_set_data (G_OBJECT (frame), "panel-applet-frame-activating", frame_act);
 
 	panel_applet_container_add (dbus_frame->priv->container,
-				    iid, NULL,
+				    screen, iid, NULL,
 				    (GAsyncReadyCallback) panel_applet_frame_dbus_activated,
 				    frame,
 				    g_variant_builder_end (&builder));
 
 	g_free (conf_path);
+	g_free (settings_path);
 	g_free (background);
 
 	return TRUE;
