@@ -29,18 +29,19 @@
 
 #include <glib/gi18n.h>
 
-#include <gconf/gconf.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 
 #include "panel-applets-manager.h"
 #include "panel-bindings.h"
-#include "panel-profile.h"
 #include "panel.h"
 #include "applet.h"
 #include "panel-marshal.h"
 #include "panel-background.h"
+#include "panel-layout.h"
 #include "panel-lockdown.h"
+#include "panel-object-loader.h"
+#include "panel-schemas.h"
 #include "panel-stock-icons.h"
 #include "xstuff.h"
 #include "panel-compatibility.h"
@@ -53,19 +54,15 @@ static void panel_applet_frame_loading_failed  (const char  *iid,
 					        PanelWidget *panel,
 					        const char  *id);
 
-static void panel_applet_frame_load            (const gchar *iid,
+static void panel_applet_frame_load_helper     (const gchar *iid,
 						PanelWidget *panel,
-						gboolean     locked,
-						int          position,
-						gboolean     exactpos,
-						const char  *id);
+						const char  *id,
+						GSettings   *settings);
 
 struct _PanelAppletFrameActivating {
-	gboolean     locked;
 	PanelWidget *panel;
-	int          position;
-	gboolean     exactpos;
 	char        *id;
+	GSettings   *settings;
 };
 
 /* PanelAppletFrame implementation */
@@ -75,7 +72,6 @@ G_DEFINE_TYPE (PanelAppletFrame, panel_applet_frame, GTK_TYPE_EVENT_BOX)
 #define PANEL_APPLET_FRAME_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_APPLET_FRAME, PanelAppletFramePrivate))
 
 #define HANDLE_SIZE 10
-#define PANEL_APPLET_PREFS_KEY "/apps/panel/applets/%s/prefs"
 
 struct _PanelAppletFramePrivate {
 	PanelWidget     *panel;
@@ -109,11 +105,16 @@ panel_applet_frame_draw (GtkWidget *widget,
 	gtk_style_context_set_state (context, gtk_widget_get_state_flags (widget));
 
 	cairo_save (cr);
-        gtk_render_handle (context, cr,
-			   frame->priv->handle_rect.x,
-			   frame->priv->handle_rect.y,
-			   frame->priv->handle_rect.width,
-			   frame->priv->handle_rect.height);
+	cairo_rectangle (cr,
+			 frame->priv->handle_rect.x,
+			 frame->priv->handle_rect.y,
+			 frame->priv->handle_rect.width,
+			 frame->priv->handle_rect.height);
+	cairo_clip (cr);
+	gtk_render_handle (context, cr,
+			   0, 0,
+			   gtk_widget_get_allocated_width (widget),
+			   gtk_widget_get_allocated_height (widget));
 	cairo_restore (cr);
 
 	gtk_style_context_restore (context);
@@ -346,6 +347,7 @@ panel_applet_frame_button_changed (GtkWidget      *widget,
 {
 	PanelAppletFrame *frame;
 	gboolean          handled = FALSE;
+	guint             modifiers;
 	GdkDisplay       *display;
 	GdkDevice        *pointer;
 	GdkDeviceManager *device_manager;
@@ -358,15 +360,18 @@ panel_applet_frame_button_changed (GtkWidget      *widget,
 	if (event->window != gtk_widget_get_window (widget))
 		return FALSE;
 
+	modifiers = event->state & GDK_MODIFIER_MASK;
+
 	switch (event->button) {
 	case 1:
 	case 2:
 		if (button_event_in_rect (event, &frame->priv->handle_rect)) {
-			if (event->type == GDK_BUTTON_PRESS ||
-			    event->type == GDK_2BUTTON_PRESS) {
+			if ((event->type == GDK_BUTTON_PRESS ||
+			     event->type == GDK_2BUTTON_PRESS) &&
+			    modifiers == panel_bindings_get_mouse_button_modifier_keymask ()){
 				panel_widget_applet_drag_start (
 					frame->priv->panel, GTK_WIDGET (frame),
-					PW_DRAG_OFF_CURSOR, event->time);
+					event->time);
 				handled = TRUE;
 			} else if (event->type == GDK_BUTTON_RELEASE) {
 				panel_widget_applet_drag_end (frame->priv->panel);
@@ -377,10 +382,6 @@ panel_applet_frame_button_changed (GtkWidget      *widget,
 	case 3:
 		if (event->type == GDK_BUTTON_PRESS ||
 		    event->type == GDK_2BUTTON_PRESS) {
-			guint modifiers;
-
-			modifiers = event->state & GDK_MODIFIER_MASK;
-
 			display = gtk_widget_get_display (widget);
 			device_manager = gdk_display_get_device_manager (display);
 			pointer = gdk_device_manager_get_client_pointer (device_manager);
@@ -413,9 +414,6 @@ panel_applet_frame_finalize (GObject *object)
 
 	panel_applets_manager_factory_deactivate (frame->priv->iid);
 
-	panel_lockdown_notify_remove (G_CALLBACK (panel_applet_frame_sync_menu_state),
-				      frame);
-
 	g_free (frame->priv->iid);
 	frame->priv->iid = NULL;
 
@@ -457,26 +455,23 @@ panel_applet_frame_init_properties (PanelAppletFrame *frame)
 	PANEL_APPLET_FRAME_GET_CLASS (frame)->init_properties (frame);
 }
 
-void
-panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
+static void
+panel_applet_frame_sync_menu_state (PanelLockdown *lockdown,
+				    gpointer       user_data)
 {
-	PanelWidget *panel_widget;
-	gboolean     locked_down;
-	gboolean     locked;
-	gboolean     lockable;
-	gboolean     movable;
-	gboolean     removable;
+	PanelAppletFrame *frame = PANEL_APPLET_FRAME (user_data);
+	PanelWidget      *panel_widget;
+	gboolean          locked_down;
+	gboolean          movable;
+	gboolean          removable;
 
 	panel_widget = PANEL_WIDGET (gtk_widget_get_parent (GTK_WIDGET (frame)));
 
 	movable = panel_applet_can_freely_move (frame->priv->applet_info);
-	removable = panel_profile_id_lists_are_writable ();
-	lockable = panel_applet_lockable (frame->priv->applet_info);
+	removable = panel_layout_is_writable ();
+	locked_down = panel_lockdown_get_panels_locked_down_s ();
 
-	locked = panel_widget_get_applet_locked (panel_widget, GTK_WIDGET (frame));
-	locked_down = panel_lockdown_get_locked_down ();
-
-	PANEL_APPLET_FRAME_GET_CLASS (frame)->sync_menu_state (frame, movable, removable, lockable, locked, locked_down);
+	PANEL_APPLET_FRAME_GET_CLASS (frame)->sync_menu_state (frame, movable, removable, locked_down);
 }
 
 void
@@ -565,23 +560,26 @@ _panel_applet_frame_activated (PanelAppletFrame           *frame,
 	frame->priv->panel = frame_act->panel;
 	gtk_widget_show_all (GTK_WIDGET (frame));
 
-	info = panel_applet_register (GTK_WIDGET (frame), GTK_WIDGET (frame),
-				      NULL, frame->priv->panel,
-				      frame_act->locked, frame_act->position,
-				      frame_act->exactpos, PANEL_OBJECT_APPLET,
-				      frame_act->id);
+	info = panel_applet_register (GTK_WIDGET (frame), frame->priv->panel,
+				      PANEL_OBJECT_APPLET, frame_act->id,
+				      frame_act->settings,
+				      GTK_WIDGET (frame), NULL);
 	frame->priv->applet_info = info;
 
 	panel_widget_set_applet_size_constrained (frame->priv->panel,
 						  GTK_WIDGET (frame), TRUE);
 
-	panel_applet_frame_sync_menu_state (frame);
+
+	panel_lockdown_on_notify (panel_lockdown_get (),
+				  NULL,
+				  G_OBJECT (frame),
+				  panel_applet_frame_sync_menu_state,
+				  frame);
+	panel_applet_frame_sync_menu_state (panel_lockdown_get (), frame);
+
 	panel_applet_frame_init_properties (frame);
 
-	panel_lockdown_notify_add (G_CALLBACK (panel_applet_frame_sync_menu_state),
-				   frame);
-
-	panel_applet_stop_loading (frame_act->id);
+	panel_object_loader_stop_loading (frame_act->id);
 	panel_applet_frame_activating_free (frame_act);
 }
 
@@ -685,31 +683,31 @@ panel_applet_frame_reload_response (GtkWidget        *dialog,
 		PanelWidget *panel;
 		char        *iid;
 		char        *id = NULL;
-		int          position = -1;
-		gboolean     locked = FALSE;
+		GSettings   *settings = NULL;
 
 		panel = frame->priv->panel;
 		iid   = g_strdup (frame->priv->iid);
 
 		if (info) {
 			id = g_strdup (info->id);
-			position  = panel_applet_get_position (info);
-			locked = panel_widget_get_applet_locked (panel, info->widget);
+			settings = panel_applet_get_settings (info);
+			g_object_ref (settings);
 			panel_applet_clean (info);
 		}
 
-		panel_applet_frame_load (iid, panel, locked,
-					 position, TRUE, id);
+		panel_applet_frame_load_helper (iid, panel, id, settings);
 
 		g_free (iid);
 		g_free (id);
+		if (settings)
+			g_object_unref (settings);
 
 	} else if (info) {
 		/* if we can't write to applets list we can't really delete
 		   it, so we'll just ignore this.  FIXME: handle this
 		   more correctly I suppose. */
-		if (panel_profile_id_lists_are_writable ())
-			panel_profile_delete_object (info);
+		if (panel_layout_is_writable ())
+			panel_layout_delete_object (panel_applet_get_id (info));
 	}
 
 	g_object_unref (frame);
@@ -785,7 +783,7 @@ _panel_applet_frame_applet_remove (PanelAppletFrame *frame)
 	info = frame->priv->applet_info;
 	frame->priv->applet_info = NULL;
 
-	panel_profile_delete_object (info);
+	panel_layout_delete_object (panel_applet_get_id (info));
 }
 
 void
@@ -799,22 +797,9 @@ _panel_applet_frame_applet_move (PanelAppletFrame *frame)
 
 	panel_widget_applet_drag_start (PANEL_WIDGET (parent),
 					widget,
-					PW_DRAG_OFF_CENTER,
 					GDK_CURRENT_TIME);
 }
 
-void
-_panel_applet_frame_applet_lock (PanelAppletFrame *frame,
-				 gboolean          locked)
-{
-	PanelWidget *panel_widget = PANEL_WIDGET (gtk_widget_get_parent (GTK_WIDGET (frame)));
-
-	if (panel_widget_get_applet_locked (panel_widget, GTK_WIDGET (frame)) == locked)
-		return;
-
-	panel_applet_toggle_locked (frame->priv->applet_info);
-}
-
 /* Generic methods */
 
 static GSList *no_reload_applets = NULL;
@@ -827,6 +812,7 @@ enum {
 static void
 panel_applet_frame_activating_free (PanelAppletFrameActivating *frame_act)
 {
+	g_object_unref (frame_act->settings);
 	g_free (frame_act->id);
 	g_slice_free (PanelAppletFrameActivating, frame_act);
 }
@@ -849,22 +835,30 @@ panel_applet_frame_activating_get_size (PanelAppletFrameActivating *frame_act)
 	return frame_act->panel->sz;
 }
 
-gboolean
-panel_applet_frame_activating_get_locked (PanelAppletFrameActivating *frame_act)
-{
-	return frame_act->locked;
-}
-
 gboolean
 panel_applet_frame_activating_get_locked_down (PanelAppletFrameActivating *frame_act)
 {
-	return panel_lockdown_get_locked_down ();
+	return panel_lockdown_get_panels_locked_down_s ();
+}
+
+gchar *
+panel_applet_frame_activating_get_settings_path (PanelAppletFrameActivating *frame_act)
+{
+        char *path;
+        char *path_instance;
+
+        g_object_get (frame_act->settings, "path", &path, NULL);
+        path_instance = g_strdup_printf ("%s%s", path,
+                                         PANEL_LAYOUT_OBJECT_CONFIG_SUFFIX);
+        g_free (path);
+
+	return path_instance;
 }
 
 gchar *
 panel_applet_frame_activating_get_conf_path (PanelAppletFrameActivating *frame_act)
 {
-	return g_strdup_printf (PANEL_APPLET_PREFS_KEY, frame_act->id);
+	return panel_layout_object_get_gconf_path (frame_act->id);
 }
 
 static void
@@ -875,8 +869,8 @@ panel_applet_frame_loading_failed_response (GtkWidget *dialog,
 	gtk_widget_destroy (dialog);
 
 	if (response == LOADING_FAILED_RESPONSE_DELETE &&
-	    !panel_lockdown_get_locked_down () &&
-	    panel_profile_id_lists_are_writable ()) {
+	    !panel_lockdown_get_panels_locked_down_s () &&
+	    panel_layout_is_writable ()) {
 		GSList *item;
 
 		item = g_slist_find_custom (no_reload_applets, id,
@@ -887,7 +881,7 @@ panel_applet_frame_loading_failed_response (GtkWidget *dialog,
 								 item);
 		}
 
-		panel_profile_remove_from_list (PANEL_GCONF_APPLETS, id);
+		panel_layout_delete_object (id);
 	}
 
 	g_free (id);
@@ -905,7 +899,7 @@ panel_applet_frame_loading_failed (const char  *iid,
 	no_reload_applets = g_slist_prepend (no_reload_applets,
 					     g_strdup (id));
 
-	locked_down = panel_lockdown_get_locked_down ();
+	locked_down = panel_lockdown_get_panels_locked_down_s ();
 
 	problem_txt = g_strdup_printf (_("The panel encountered a problem "
 					 "while loading \"%s\"."),
@@ -950,16 +944,14 @@ panel_applet_frame_loading_failed (const char  *iid,
 
 	/* Note: this call will free the memory for id, so the variable should
 	 * not get accessed afterwards. */
-	panel_applet_stop_loading (id);
+	panel_object_loader_stop_loading (id);
 }
 
 static void
-panel_applet_frame_load (const gchar *iid,
-			 PanelWidget *panel,
-			 gboolean     locked,
-			 int          position,
-			 gboolean     exactpos,
-			 const char  *id)
+panel_applet_frame_load_helper (const gchar *iid,
+				PanelWidget *panel,
+				const char  *id,
+				GSettings   *settings)
 {
 	PanelAppletFrameActivating *frame_act;
 
@@ -969,21 +961,19 @@ panel_applet_frame_load (const gchar *iid,
 
 	if (g_slist_find_custom (no_reload_applets, id,
 				 (GCompareFunc) strcmp)) {
-		panel_applet_stop_loading (id);
+		panel_object_loader_stop_loading (id);
 		return;
 	}
 
-	if (panel_lockdown_is_applet_disabled (iid)) {
-		panel_applet_stop_loading (id);
+	if (panel_lockdown_is_applet_disabled (panel_lockdown_get (), iid)) {
+		panel_object_loader_stop_loading (id);
 		return;
 	}
 
 	frame_act = g_slice_new0 (PanelAppletFrameActivating);
-	frame_act->locked   = locked;
 	frame_act->panel    = panel;
-	frame_act->position = position;
-	frame_act->exactpos = exactpos;
 	frame_act->id       = g_strdup (id);
+	frame_act->settings = g_object_ref (settings);
 
 	if (!panel_applets_manager_load_applet (iid, frame_act)) {
 		panel_applet_frame_loading_failed (iid, panel, id);
@@ -992,47 +982,36 @@ panel_applet_frame_load (const gchar *iid,
 }
 
 void
-panel_applet_frame_load_from_gconf (PanelWidget *panel_widget,
-				    gboolean     locked,
-				    int          position,
-				    const char  *id)
+panel_applet_frame_load (PanelWidget *panel_widget,
+			 const char  *id,
+			 GSettings   *settings)
 {
 	gchar *applet_iid;
 
 	g_return_if_fail (panel_widget != NULL);
 	g_return_if_fail (id != NULL);
 
-	applet_iid = panel_compatibility_get_applet_iid (id);
+	applet_iid = panel_compatibility_get_applet_iid (settings, id);
 	if (!applet_iid) {
-		panel_applet_stop_loading (id);
+		panel_object_loader_stop_loading (id);
 		return;
 	}
 
-	panel_applet_frame_load (applet_iid, panel_widget,
-				 locked, position, TRUE, id);
+	panel_applet_frame_load_helper (applet_iid, panel_widget, id, settings);
 
 	g_free (applet_iid);
 }
 
 void
-panel_applet_frame_create (PanelToplevel *toplevel,
-			   int            position,
-			   const char    *iid)
+panel_applet_frame_create (PanelToplevel       *toplevel,
+			   PanelObjectPackType  pack_type,
+			   int                  pack_index,
+			   const char          *iid)
 {
-	GConfClient *client;
-	const char  *key;
-	char        *id;
-
 	g_return_if_fail (iid != NULL);
 
-	client =  panel_gconf_get_client ();
-
-	id = panel_profile_prepare_object (PANEL_OBJECT_APPLET, toplevel, position, FALSE);
-
-	key = panel_gconf_full_key (PANEL_GCONF_APPLETS, id, "applet_iid");
-	gconf_client_set_string (client, key, iid, NULL);
-
-	panel_profile_add_to_list (PANEL_GCONF_APPLETS, id);
-
-	g_free (id);
+	panel_layout_object_create (PANEL_OBJECT_APPLET,
+				    iid,
+				    panel_toplevel_get_id (toplevel),
+				    pack_type, pack_index);
 }
