@@ -32,7 +32,6 @@
 #include "panel-icon-names.h"
 
 static GtkIconSize panel_menu_icon_size = 0;
-static GtkIconSize panel_menu_bar_icon_size = 0;
 
 GtkIconSize
 panel_menu_icon_get_size (void)
@@ -40,12 +39,6 @@ panel_menu_icon_get_size (void)
 	return panel_menu_icon_size;
 }
 
-GtkIconSize
-panel_menu_bar_icon_get_size (void)
-{
-	return panel_menu_bar_icon_size;
-}
-
 typedef struct {
 	char *stock_id;
 	char *icon;
@@ -127,10 +120,6 @@ panel_init_stock_icons_and_items (void)
 						       PANEL_DEFAULT_MENU_ICON_SIZE,
 						       PANEL_DEFAULT_MENU_ICON_SIZE);
 
-	panel_menu_bar_icon_size = gtk_icon_size_register ("panel-foobar",
-							   PANEL_DEFAULT_MENU_ICON_SIZE,
-							   PANEL_DEFAULT_MENU_ICON_SIZE);
-
 	factory = gtk_icon_factory_new ();
 	gtk_icon_factory_add_default (factory);
 
